;
; IDL Event Callback Procedures
; MSP_SCT_eventcb
;
; Generated on:	04/02/2015 13:42.44
;   v1.0	DPC	20150402
;	v1.1	DPC 20150407	bug fixes
;
;-----------------------------------------------------------------
;
pro MSP_SCT_Realize, wWidget
	;
	;	get key window information
	;
	WIDGET_CONTROL, wWidget, GET_UVALUE=PTR_main
	;
;	;	need to pass an event structure:
;	;
	event = {top: wWidget}
	;
	;	fetch plot window WID
	;
	plot_wid = WIDGET_INFO(wWidget, FIND_BY_UNAME="Plot_Area")
	(*PTR_main).plot_wid = plot_wid
	;
	;	and turn off the events generated in the plot window
	;
	WIDGET_CONTROL, plot_wid, DRAW_MOTION_EVENTS=0
	WIDGET_CONTROL, plot_wid, DRAW_BUTTON_EVENTS=0
	;
	;	set color table
	;
	device, decomposed=0
	tvlct, [0, 255, 255,   0,   0, 100, 255, 128, 255], $
		   [0, 255,   0, 255,   0, 200,   0, 128, 255], $
		   [0, 255,   0,   0, 255, 200, 255,   0,   0], 10
	color_names = ['black','white','red','green','blue','cyan','magenta','olive','yellow']
	colors = [10,11,12,13,14,15,16,17,18]
	;
	(*PTR_main).color_names = color_names
	(*PTR_main).colors = colors
	;
	WIDGET_CONTROL, wWidget, SET_UVALUE=PTR_main
	;
	;	erase it and write a message there
	;
	plot_print,event,['Mimir Software Package:',$
		'Spectra Combining Tool','Initialized']
	;
end
;
;-----------------------------------------------------------------
;
pro MSP_SCT_PostCreation, wWidget, _EXTRA=_VWBExtra_
;
    ;   set up and clear main structure
    ;
    ;
    main_struct =  {plot_wid:       0L,      		$         ; wid of plotting window
                    ;
                    draw_button_focus:	'none',		$		  ; which routine turned on button events in the draw window
                    										  ;    none = none yet; left, right, start, stop, x, y
                    draw_motion_focus:	'none',		$		  ; same
                    cursor_x:		0.0,			$		  ; cursor x value buffer
                    cursor_y:		0.0,			$		  ; cursor y value buffer
                    ;
                    cursor_save_x:	0.0,			$		  ; cursor x value, in microns
                    cursor_save_y:	0.0,			$		  ; cursor y value, in flux
                    cursor_save_spectrum:	0L,		$	      ; spectrum number pointed to
                    ;
                    draw_x_size:	0.0,			$		  ; full size of the draw window along x, in pixels
                    draw_y_size:	0.0,			$		  ;
                    plot_x_offset:	0.0,			$		  ; offset of LL x of plot axis, in pixels, from draw window corner
                    plot_y_offset:	0.0,			$		  ;           LL y
                    plot_x_size:	0.0,			$		  ; length of plotted x axis, in pixels
                    plot_y_size:	0.0,			$		  ;                   y
                    active_offsets:	0L,				$		  ; if set, actively update plot with offsets
                    ;
                    plot_x_scale:	0.0,			$		  ; x plot scale, in microns per pixel
                    plot_y_scale:	0.0,			$		  ; y plot scale, in flux per pixel
                    ;
                    color_names:	strarr(9),		$		  ; names of plotting colors
                    colors:			intarr(9),		$		  ; corresponding color numbers
                    ;
                    nfiles:			0L,				$		  ; number of files read
                    nfiles_max:		50L,			$		  ; file number limit
                    filenames:		strarr(50),		$		  ; filenames of the spectrum read, stripped of directory name
                    dirname:		'',				$		  ; directory name of the spectrum read
                    ;
                    spectrum_ready: 0L,				$		  ; if 1, combined spectrum ready for plotting, else not
					spectrum_nch:	0L,				$		  ; number of channels in the spectrum
                    spectrum_wave:	fltarr(5000),	$		  ; space for the wavelengths of each channel
                    spectrum_flux:	fltarr(5000),	$		  ; space for flux in each channel
                    spectrum_sflux: fltarr(5000),	$		  ; space for flux_uncertainty in each channel
                    spectrum_color:	intarr(5000),	$		  ; color to plot each channel in the spectrum (white = normal; red = overlap)
                    ;
                    header:			PTR_NEW(/alloc),$		  ; FITS header
                    ;
                    input_nch:		intarr(50),		$		  ; number of channels in each input spectrum
                    input_wave:		fltarr(5000,50),$		  ; input spectra - wavelengths
                    input_flux:		fltarr(5000,50),$		  ;               - fluxes
                    input_sflux:	fltarr(5000,50),$		  ;               - flux uncertainties
                    input_center:	fltarr(50),		$		  ; input spectrum center wavelength
                    input_base:		fltarr(50),		$		  ; input spectrum baseline at 0 or 1?
                    input_offset:	fltarr(50),		$		  ; vertical offset to ADD when combinging
                    ;
                    default_left:		0.0,		$		  ; initial left plot limit, in microns
                    default_right:		0.0,		$		  ; initial right plot limit, in microns
                    default_top:		0.0,		$		  ; initial top plot limit. If zero, use max+5%
                    default_bottom:		0.0,		$		  ; initial bottom plot limit. If zero, use min-5%
                    ;
                    left:				0.0,		$		  ; current left plot limit, in microns
                    right:			    0.0,		$		  ; current right plot limit, in microns
                    top:				0.0,		$		  ; current top plot limit. If zero, use max+5%
                    bottom:				0.0,		$		  ; current bottom plot limit. If zero, use min-5%
                    ;
                    left_next:			0.0,		$		  ; requested next left limit wavelength
                    right_next:			0.0,		$		  ; requested next right limit wavelength
                    top_next:			0.0,		$		  ;
                    bottom_next:		0.0,		$
                    ;
                    plot_zero_line:		0L,			$		  ; if 1 - plot dashed zero line
                    plot_unity_line:	0L,			$		  ; if 1 - plot dashed unity line
                    plot_uncertainties:	0L,			$		  ; if 1 - plot flux uncertainties also
                    rescale_y_flag:		0L,			$		  ; if 1 - rescale y values on replot
                    this_offset:		0.0,		$		  ; current offset
                    ;
                    ;	plot stuff
                    ;
                    p_ch:				1.0,		$		  ; plotting charactersize
                    p_color:		  'white'		$
                    }
    ;
	;
	;------------------------------------------------------------------------
    ;
    PTR_main = PTR_NEW(main_struct,/NO_COPY)
    ;
    WIDGET_CONTROL, wWidget, SET_UVALUE=PTR_main
end
;
;----------------------------------------------------------------------
;
function get_color, Event, name
	;
	;	returns color index, given name
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_main
	;
	ind = WHERE(name eq (*PTR_main).color_names,n)
	if(n eq 1) then begin
		clr = ind[0]+(*PTR_main).colors[0]
	endif else begin
		clr = -1
	endelse
	;
	return, clr
end
;
;-----------------------------------------------------------------
;
pro plot_print,event,str
	;
	;	prints the str into the plotting window
	;	if str is an array, stacks the strings on the plot window
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_main
	;
	nrows = N_ELEMENTS(str)
	;
	if(nrows eq 1) then begin
		this_str = [str]
		y = [0.5]
	endif else begin
		this_str = str
		y = findgen(nrows)*(-0.5/float(nrows-1))+0.75
	endelse
	;
	ERASE
	plot, [0,1],[0,1],/nodata,xs=4,ys=4
	;
	for i = 0, nrows-1 do begin
		;
		XYoutS, 0.5,y[i],str[i],align=0.5,charsize=1.5
	endfor
	;
end
;
;-----------------------------------------------------
;
pro Exit_Asserted, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_main
	;
	; free the pointer
	;
	PTR_FREE, PTR_main
	;
	WIDGET_CONTROL, Event.top, /DESTROY
end
;
;--------------------------
;
pro Select_Files, Event
	;
	;	fetch pointer to data structure
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_main
	;
	;	if the current directory is not set, then just throw up a file chooser,
	;	but if set, point the file chooser to that directory
	;
	restart:
	;
	filters = ['*.fits','*.FITS','*.fts','*.fit']
	if( (*PTR_main).dirname eq '') then begin
		filenames = Dialog_Pickfile(title='Select SpeX-type Spectrum', $
					FILTER=filters, /multiple_files)
		this_dir = FILE_DIRNAME(filenames[0])
		(*PTR_main).dirname = this_dir
	endif else begin
		filenames = Dialog_Pickfile(title='Select SpeX-type Spectrum',$
			PATH = (*PTR_main).dirname, FILTER=filters, /multiple_files)
	endelse
	;
	;	filenames selected
	;
	(*PTR_main).nfiles = N_ELEMENTS(filenames)
	;
	;	test for too many
	;
	if((*PTR_main).nfiles gt (*PTR_main).nfiles_max) then begin
		plot_print, Event, ['ERROR - Too many input files','Select no more than 50 files',$
				'Try again']
		goto, restart
	endif
	;
	;
	; read each file
	;
	wid = WIDGET_INFO(Event.top, FIND_BY_UNAME="N_Files_Textbox")
	;
	for i = 0, (*PTR_main).nfiles-1 do begin
		;
		;
		image = readfits(filenames[i],header,/silent)
		;
		;	if first image, save header
		;
		if(i eq 0) then (*(*PTR_main).header) = header
		;
		;	if, ok, load some stuff
		;
		s = size(image)
		nx = s[1]					; channel dimension
		ny = s[2]					; should be 3
		;
		if(ny ne 3 or nx lt 10) then begin
			plot_print,Event,['Bad SpeX FITS File','Skipping this file']
			goto, not_read
		endif
		;
		;	looks OK, save some stuff
		;
		WIDGET_CONTROL, wid, SET_VALUE=STRING(i+1,format='(I2)')		; number of files read
		;
		(*PTR_main).filenames[i] = FILE_BASENAME(filenames[i])
		;
		;	if first one, save directory, also
		;
		;
		(*PTR_main).input_nch[i] = nx				; channels in this spectrum
		;
		;	then, store the vectors
		;
		(*PTR_main).input_wave[0:nx-1,i] = REFORM(image[*,0])
		(*PTR_main).input_flux[0:nx-1,i] = REFORM(image[*,1])
		(*PTR_main).input_sflux[0:nx-1,i] = REFORM(image[*,2])
		;
		mc = MOMENT( (*PTR_main).input_wave[0:nx-1,i] )
		(*PTR_main).input_center[i] = mc[0]						; center wavelength
		;
		vm = median_filtered_mean((*PTR_main).input_flux[0:nx-1,i])
		vd = vm[0] - 0.5
		if(vd gt 0.0) then vv = 1.0 else vv = 0.0				; baseline value
		(*PTR_main).input_base[i] = vv
		;
		;   clean NaNs - ** hold off on clearing them for now
		;
;		ind_f = WHERE(~FINITE((*(*PTR_main).spectrum_flux)[*]),count)
;		if(count ne 0) then begin
;	;	    print,"Found NaNs, count = ",count
;		    ind_good = WHERE(FINITE((*(*PTR_main).spectrum_flux)[*]))
;		    zz = median_filtered_mean((*(*PTR_main).spectrum_flux)[ind_good])
;		    (*(*PTR_main).spectrum_flux)[ind_f] = zz[0]
;		    (*(*PTR_main).spectrum_sflux)[ind_f] = zz[0]
;	;	    print, "reset to mean of ",zz[0]
;		endif
		;
		;	clear some buffers back to default values
		;
		(*PTR_main).left = (*PTR_main).default_left
		(*PTR_main).left_next = (*PTR_main).default_left
		;
		(*PTR_main).right = (*PTR_main).default_right
		(*PTR_main).right_next = (*PTR_main).default_right
		;
		(*PTR_main).top = (*PTR_main).default_top
		(*PTR_main).top_next = (*PTR_main).default_top
		;
		(*PTR_main).bottom = (*PTR_main).default_bottom
		(*PTR_main).bottom_next = (*PTR_main).default_bottom
		;
		not_read:
	endfor
	;
	plot_spectrum,event
end
;
;-----------------------------------------------------------------
pro Cursor_On, Event
	;
	;	turn on motion sensitivity in plot window, listen for mouse
	;	click
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_main
	;
	plot_wid = (*PTR_main).plot_wid
	;
	WIDGET_CONTROL, plot_wid, DRAW_BUTTON_EVENTS=1
	WIDGET_CONTROL, plot_wid, DRAW_MOTION_EVENTS=1
	;
	;	control passed to other routines as a result
	;
end
;
;-----------------------------------------------------------------------
;
pro Draw_Button, Event
	;
	;	if the mouse button was pressed, start saving and implementing
	;	spectral offsets.
	;
	;	if the mouse button is released, turn off the cursor watching
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_main
	;
	plot_wid = (*PTR_main).plot_wid
	;
	WIDGET_CONTROL, plot_wid, DRAW_MOTION_EVENTS=1
	WIDGET_CONTROL, plot_wid, DRAW_BUTTON_EVENTS=1	; or the mouse turns off!
	;
	; 	test for mouse press
	;
	case Event.type of
		0:	begin							;   press button
					toggle_active_offsets, Event
;					print, 'in draw-button, toggled active offsets. new value = ',(*PTR_main).active_offsets
			end
		1:	begin							; released button, leave motion events off
;				print, 'button release sensed'
			end
		2:	begin							; motion, leave motion events on
					WIDGET_CONTROL, plot_wid, DRAW_MOTION_EVENTS=1
;					print, 'motion sensed'
			end
   endcase
   ;
end
;
;-----------------------------------------------------------------
;
pro Draw_Motion, Event
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_main
	;
	plot_wid = (*PTR_main).plot_wid
	;
	;	save the X, Y position to the buffer
	;
	(*PTR_main).cursor_x = Event.x
	(*PTR_main).cursor_y = Event.y
	;
	;	which spectrum's center is this closest to?
	;
	;	 wavelength offset from left
 	;
 	lambda = (Event.x - (*PTR_main).plot_x_offset) * (*PTR_main).plot_x_scale
	lambda += (*PTR_main).left
	;
	dif = abs(lambda - (*PTR_main).input_center[0:(*PTR_main).nfiles-1])
	;
	dmin = MIN(dif,this_spectrum)
	;
	sp_wid = WIDGET_INFO(Event.top, FIND_BY_UNAME='Offset_Spectrum')
	WIDGET_CONTROL, sp_wid, SET_VALUE=STRING(this_spectrum+1,format='(I2)')
	;
	;	compute vertical offset from base
	;
	height = (Event.y - (*PTR_main).plot_y_offset) * (*PTR_main).plot_y_scale
	height += (*PTR_main).bottom - (*PTR_main).input_base[this_spectrum]
	;
	win_wid = WIDGET_INFO(Event.top, FIND_BY_UNAME='Offset_Textbox')
 	WIDGET_CONTROL, win_wid, SET_VALUE=STRING(height,format='(E19.6)')
	;
	;	save the wavelength and offset in the structure
	;
	(*PTR_main).cursor_x = lambda
	(*PTR_main).cursor_y = height
	(*PTR_main).cursor_save_spectrum = this_spectrum
	;
	;	print value to correct window and save on structure
	;
;	print, 'Motion Event - x = ',Event.x,'  y = ',Event.y,'  wavelength = ',lambda,' offset = ',height
	;
	;	if button pressed, start doing stuff
	;
	case Event.type of
		0:	begin
				toggle_active_offsets, Event							;   press button
;				print, 'active offset value = ',(*PTR_main).active_offsets
				;
				;	if one, save cursor positions
				;
				if( (*PTR_main).active_offsets eq 1) then begin
					(*PTR_main).cursor_save_x = (*PTR_main).cursor_x
					(*PTR_main).cursor_save_y = (*PTR_main).cursor_y
				endif else begin
					;
					;	if toggled to zero, turn off mouse sensitivity
					;
					WIDGET_CONTROL, plot_wid, DRAW_BUTTON_EVENTS=1
					WIDGET_CONTROL, plot_wid, DRAW_MOTION_EVENTS=1
				endelse
			end
		1:	begin							; released button, leave motion events off
;				print, 'button release sensed'
			end
		2:	begin							; motion, leave motion events on
				if((*PTR_main).active_offsets eq 1) then begin;	which spectrum?
					;
					this_spectrum = (*PTR_main).cursor_save_spectrum
					;
					;	how much offset?
					;
					this_offset = (*PTR_main).cursor_save_y  - (*PTR_main).cursor_y
					;
					;	increment this spectrum's offset by this amount
					;
					(*PTR_main).input_offset[this_spectrum] -= this_offset
					;
					WIDGET_CONTROL, Event.top, SET_UVALUE=PTR_main
					;
;					print, 'spectrum ',this_spectrum+1,' saved y = ',(*PTR_main).cursor_save_y,' bottom = ',$
;							(*PTR_main).bottom,' increment offset = ',this_offset
					;
					;	and replot
					;
					plot_spectrum, Event
					;
					wait, 0.2
					;
					;	update the y save buffer
					;
					(*PTR_main).cursor_save_y = (*PTR_main).cursor_y
					;
					;	mouse listening back on
					;
					WIDGET_CONTROL, plot_wid, DRAW_MOTION_EVENTS=1
				endif
;				print, 'motion sensed'
			end
   endcase
	;
end
;
;-----------------------------------------------------------------
;
pro Combine_Spectra, Event
	;
	;	combines all the spectra, with offsets
	;
	;	if spectra overlap, takes lower uncertainty, but colors overlap region
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_main
	;
	;	number of spectra to assemble
	;
	nfiles = (*PTR_main).nfiles
	;
	;	first, check that all wavelength steps are the same
	;	*** later, this requirement could be lifted, but requires lots more coding ***
	;
	dl_dch = fltarr(nfiles,2)			; mean, dispersion of channel to channel differences
	;
	for i = 0, nfiles-1 do begin
		;
		nch_i = (*PTR_main).input_nch[i]
		;
		dif = (*PTR_main).input_wave[1:nch_i-1,i] - (*PTR_main).input_wave[0:nch_i-2,i]
		;
		zz = MOMENT(dif)
		dl_dch[i,0] = zz[0]
		dl_dch[i,1] = sqrt(zz[1])
		;
		print, 'for file ',i+1,' internal wavelength difference mean = ',zz[0],' dispersion = ',sqrt(zz[1])
		;
	endfor
	;
	;	compare channel steps
	;
	vec = REFORM(dl_dch[*,0])
	zz = MOMENT(vec)
	;
	print,'mean channel offsets = ',zz[0],' dispersion = ',sqrt(zz[1])
	;
	mco = zz[0]
	;
	;	so long as mean / dispersion is > 10^5 we are fine
	;
	if(abs(zz[0]/sqrt(zz[1])) gt 1.0e5) then begin
		;
		;	next, find shortest, longest wavelengths
		;
		lam_0 = 1.0e6
		lam_1 = -1.0e6
		;
		for i = 0, nfiles-1 do begin
			lam_0 <= MIN((*PTR_main).input_wave[0:(*PTR_main).input_nch[i]-1,i])
			lam_1 >= MAX((*PTR_main).input_wave[0:(*PTR_main).input_nch[i]-1,i])
		endfor
		;
		;	set up output spectrum
		;
		nout_ch = FLOOR(1.5 + (lam_1 - lam_0) / mco)
		;
		(*PTR_main).spectrum_nch = nout_ch
		(*PTR_main).spectrum_wave[0:nout_ch-1] = lam_0 + findgen(nout_ch) * zz[0]
		;
		;	for every one of these channels, look at the input spectra and select the flux with
		;	the lowest non-zero flux uncertainty
		;
		;	if there are more than one at any wavelength, set the color of that spectral channel to red
		;
		total_match = 0L
		;
		for ich = 0, nout_ch-1 do begin
			this_wave = (*PTR_main).spectrum_wave[ich]
			;
			nmatch = 0
			(*PTR_main).spectrum_color[ich] = 0					; hold match count here for now
			;
			save_sflux = 1.0e20
			save_flux = 0.0
			;
			for i = 0, nfiles-1 do begin
				;
				;	any channels within 10% (of mean channel offset) at this wavelength?
				;
				ind = WHERE( abs( ((*PTR_main).input_wave[0:(*PTR_main).input_nch[i]-1,i] - this_wave)/this_wave) $
						lt 0.1*mco,nind)
				;
				if(nind gt 0) then begin
					nmatch++
					;
					if(nind eq 1) then begin
						;
						;	test that this input spectrum's channel flux uncertainty is non-zero (and non unity) and finite
						;
						sflux = (*PTR_main).input_sflux[ind[0],i]
						if(FINITE(sflux) and sflux ne 0.0 and sflux ne 1.0) then begin
							;
							;	ok, got a value, save flux, sflux if this sflux lower than before
							;
							if(sflux le save_sflux) then begin
								save_sflux = sflux
								save_flux = (*PTR_main).input_flux[ind[0],i] + (*PTR_main).input_offset[i]	; remember to add offset
							endif
						endif
					endif else begin
						print, 'ERROR - found multiple matching channels. Stopping.'
						stop
					endelse
				endif
			endfor
			;
			;	ok, evaluate the catch -
			;
			if(nmatch gt 0) then begin
				if(nmatch eq 1) then begin
					(*PTR_main).spectrum_color[ich] = 1
					(*PTR_main).spectrum_flux[ich] = save_flux
					(*PTR_main).spectrum_sflux[ich] = save_sflux
				endif else begin
					;
					;	was more than one, set color of spectral channel to red
					;
					(*PTR_main).spectrum_color[ich] = 2
					(*PTR_main).spectrum_flux[ich] = save_flux
					(*PTR_main).spectrum_sflux[ich] = save_sflux
				endelse
			endif else begin
				;
				;	no data - put deleted data in for now
				;
				(*PTR_main).spectrum_flux[ich] = -1.0e6
				(*PTR_main).spectrum_sflux[ich] = -1.0e6
			endelse
			;
			total_match += nmatch
		endfor
		;
		;	if total_match zero, stop
		;
		if(total_match eq 0) then begin
			print, 'ERROR - no channels matched. stopping.'
			stop
		endif
		;
		;	ok, evaluate the mean flux values that are not deleted to
		;		decide how to assign the deleted ones (to zero or 1)
		;
		ind_good = WHERE( (*PTR_main).spectrum_flux[0:nout_ch-1] gt -1.0e5,ngood,$
					COMPLEMENT=ind_bad,NCOMPLEMENT=nbad)
		if(ngood gt 0) then begin
			zz = median_filtered_mean( (*PTR_main).spectrum_flux[ind_good])
			;
			ctr = zz[0] - 0.5
			if(ctr gt 0) then base = 1.0 else base = 0.0
			;
			if(nbad gt 0) then begin
				(*PTR_main).spectrum_flux[ind_bad] = base
				(*PTR_main).spectrum_sflux[ind_bad] = 0.0
			endif
		endif
		;
		;	the spectrum is assembled and read to plot
		;
		(*PTR_main).spectrum_ready = 1
		;
		plot_full_spectrum, Event
		;
	endif
end
;
;-----------------------------------------------------------------
;
pro Enable_Offsetting, Event
	;
	;	turn on cursor sensing in plot area
	;
	;	if mouse button pushed, identify which spectrum is selected
	;
	;	as mouse moves, offset that spectrum by that amount
	;
	;	when mouse button is released, keep that offset
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_main
	;
	(*PTR_main).active_offsets = 0
	;
	cursor_on, Event
	;
end
;
;-----------------------------------------------------------------
;
pro Save_Spectrum, Event
	;
	;	activates the file chooser to allow user to write combined
	;	spectrum to disk
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_main
	;
	;	go to current directory to open file chooser
	;
	filters = ['*.fits','*.FITS','*.fts']
	outputfile = Dialog_Pickfile(Title='Name for Output FITS Spectrum File',$
			PATH=(*PTR_main).dirname, FILTER=filters, DEFAULT_EXTENSION='fits')
	;
	;	update the header with info about the spectra combined
	;
	this_header = (*(*PTR_main).header)
	FXADDPAR, this_header, "COMMENT", "Combined Spectrum generated by MSP_SCT, from"
	;
	;	list the input spectra
	;
	for i = 0, (*PTR_main).nfiles-1 do begin
		FXADDPAR, this_header, "COMMENT", "Input Spectrum "+STRING(i+1,format='(I2)')+$
						' = '+(*PTR_main).filenames[i]
	endfor
	;
	FXADDPAR, this_header, "CRVAL1",(*PTR_main).spectrum_wave[0],"First channel wavelength"
	;
	;	create new output array
	;
	nch = (*PTR_main).spectrum_nch
	;
	array = fltarr(nch, 3)
	;
	array[*,0] = (*PTR_main).spectrum_wave[0:nch-1]
	array[*,1] = (*PTR_main).spectrum_flux[0:nch-1]
	array[*,2] = (*PTR_main).spectrum_sflux[0:nch-1]
	;
	writefits, outputfile, array, this_header
	;
	;	toss up a message to inform use that this worked
	;
	result = Dialog_Message("FITS File "+FILE_BASENAME(outputfile)+" Written",/INFO)
	;
end
;
;-----------------------------------------------------------------
;
; Empty stub procedure used for autoloading.
;
pro MSP_SCT_eventcb
end
;
;-------------------------------------------------------------------
;
pro plot_spectrum, Event
	;
	;	plots the specta, by color
	;
	;	includes adding offsets
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_main
	;
	;
	color_order = [11, 12, 13, 14, 17, 16]	; cycle through the colors by spectrum
	this_color = 0
	;
	;   initialize - left, right, top, bottom - based on spectral wavelengths and fluxes
	;
	for i = 0, (*PTR_main).nfiles-1 do begin
		;
		this_nch = (*PTR_main).input_nch[i]
		;
		if(i eq 0) then begin
			all_wave = (*PTR_main).input_wave[0:this_nch-1,i]
			all_flux = (*PTR_main).input_flux[0:this_nch-1,i]
		endif else begin
			all_wave = [all_wave, (*PTR_main).input_wave[0:this_nch-1,i]]
			all_flux = [all_flux, (*PTR_main).input_flux[0:this_nch-1,i]]
		endelse
	endfor
	;
	max_nch = N_ELEMENTS(all_wave)
	spec_nch = (*PTR_main).spectrum_nch
	;
	;	if x next limits set to zero, find limits, else use saved values
	;
	if((*PTR_main).left eq 0.0 or (*PTR_main).spectrum_ready eq 0) then begin
		xmin = min(all_wave)
		(*PTR_main).left = xmin
	endif else begin
		xmin = (*PTR_main).left
		ind = WHERE((*PTR_main).spectrum_wave[0:spec_nch-1] le xmin,nind)
		if(nind gt 0) then begin
			ich0 = ind[nind-1]				; for the combined spectrum
		endif
	endelse
	;
	if((*PTR_main).right eq 0.0 or (*PTR_main).spectrum_ready eq 0) then begin
		xmax = max(all_wave)
		(*PTR_main).right = xmax
	endif else begin
		xmax = (*PTR_main).right
		ind = WHERE((*PTR_main).spectrum_wave[0:spec_nch-1] ge xmax,nind)
		if(nind gt 0) then begin
			ich1 = ind[0]
		endif
	endelse
	;
	replot = 1
	if((*PTR_main).bottom eq 0.0 or (*PTR_main).spectrum_ready eq 0) then begin
		if((*PTR_main).rescale_y_flag eq 1L) then ymin = MIN((*PTR_main).spectrum_flux[ich0:ich1]) else ymin = MIN(all_flux)
		(*PTR_main).bottom = ymin
		replot = 0
	endif else begin
		ymin = (*PTR_main).bottom
	endelse
	;
	if((*PTR_main).top eq 0.0 or (*PTR_main).spectrum_ready eq 0) then begin
		if((*PTR_main).rescale_y_flag eq 1L) then ymax = MAX((*PTR_main).spectrum_flux[ich0:ich1]) else ymax = MAX(all_flux)
		(*PTR_main).top = ymax
		replot = 0
	endif else begin
		ymax = (*PTR_main).top
	endelse
	;
;	print, "Min, Max wavelegths = ",xmin, xmax
;	print, "Min, Max fluxes = ",ymin, ymax
;	if(ymin eq !VALUES.F_NAN or ymax eq !VALUES.F_NAN) then begin
;;	    print,"extreme flux values found...resetting"
;	    zz = median_filtered_mean(flux)
;	    ymin = zz[0] - 10.0 * zz[1]
;	    ymax = zz[0] + 10.0 * zz[1]
;	endif
	;
	;	if original spectrum, provide a bit of room at top and bottom,
	;	but if a replot, skip this
	;
	dy = (ymax - ymin)
	;
	if(replot eq 0) then begin
		ymax = ymax + 0.05 * dy
		ymin = ymin - 0.05 * dy
	endif
	;
	(*PTR_main).top = ymax
	(*PTR_main).bottom = ymin
	;
	;		plot limits found. Plot boundary, then loop through spectra
	;
	xxplot = [xmin, xmax]
	yyplot = [ymin, ymax]
	;
	plot, xxplot, yyplot, XRANGE=[xmin, xmax], YRANGE=[ymin, ymax], XSTYLE=1, $
		color=11, background=10, thick=1, YSTYLE=1, LINESTYLE=0, PSYM=10, $
		xtitle='Wavelength [microns]',/nodata
	;
	for i = 0, (*PTR_main).nfiles-1 do begin
		;
		;   load plot vectors
		;
		this_nch = (*PTR_main).input_nch[i]
		;
		xplot = (*PTR_main).input_wave[0:this_nch-1,i]
		yplot = (*PTR_main).input_flux[0:this_nch-1,i]
		splot = (*PTR_main).input_sflux[0:this_nch-1,i]
		;
		;	fetch this spectrum's offset
		;
		yoff = (*PTR_main).input_offset[i]
		;
		yplot += yoff
		;
		;   initial plot
		;
		;cursor, xxx, yyy, /NOWAIT     ; to clear cursor buffer
		;
		;replot:
		;
		;xmin = MIN(xplot)
		;xmax = MAX(xplot)
		;ymin = MIN(yplot)
		;ymax = MAX(yplot)
		;

		oplot, xplot, yplot,thick=2, psym=-3,symsize=0.003, color=color_order[this_color]
		;
		this_color = (this_color + 1) mod N_ELEMENTS(color_order)
		;
		;	if the flag for the flux uncertainty spectrum is also set, draw a light version of it
		;
		if((*PTR_main).plot_uncertainties eq 1L) then begin
			oplot, xplot, splot, thick=1, color=get_color(Event, 'yellow')
		endif
		;
		;	trap, save plot limits
		;
	endfor
	;
	clip = !P.clip
	(*PTR_main).plot_x_offset = clip[0]					; starting x pixel
	(*PTR_main).plot_y_offset = clip[1]					; starting y pixel
	(*PTR_main).plot_x_size = clip[2] - clip[0] + 1		; x size of plot region
	(*PTR_main).plot_y_size = clip[3] - clip[1] + 1		; y size of plot region
	;
	;	plot scale factors
	;
	(*PTR_main).plot_x_scale = ( (*PTR_main).right - (*PTR_main).left ) / (*PTR_main).plot_x_size
	(*PTR_main).plot_y_scale = ( (*PTR_main).top - (*PTR_main).bottom ) / (*PTR_main).plot_y_size
	;
	;   if unity is between ymax and ymin, plot continuum line
	;
	if(ymax gt 1. and ymin lt 1. and (*PTR_main).plot_unity_line eq 1L) then begin
	    xz = xxplot
	    yz = [1., 1.]
	    oplot, xz, yz, color=get_color(event,'green'),linestyle=2
	endif
	;
	;	dashed zero line, if flag set and zero contained in plot
	;
	if(ymax gt 0. and ymin lt 0. and (*PTR_main).plot_zero_line eq 1L) then begin
	    xz = xxplot
	    yz = [0., 0.]
	    oplot, xz, yz, color=get_color(event,'blue'),linestyle=2
	endif
	;
end
;
;------------------------------------------------------------------
;
pro plot_full_spectrum, Event
	;
	;	plots the specta, by color
	;
	;	includes adding offsets
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_main
	;
	;
	color_order = [14,11,12]	; channel colors set by spectrum
	;
	;   initialize - left, right, top, bottom - based on spectral wavelengths and fluxes
	;
	this_nch = (*PTR_main).spectrum_nch
	all_wave = (*PTR_main).spectrum_wave[0:this_nch-1]
	all_flux = (*PTR_main).spectrum_flux[0:this_nch-1]
	all_sflux = (*PTR_main).spectrum_sflux[0:this_nch-1]
	;
	max_nch = N_ELEMENTS(all_wave)
	spec_nch = this_nch
	;
	;	if x next limits set to zero, find limits, else use saved values
	;
	if((*PTR_main).left eq 0.0) then begin
		xmin = min(all_wave)
		(*PTR_main).left = xmin
	endif else begin
		xmin = (*PTR_main).left
		ind = WHERE((*PTR_main).spectrum_wave[0:spec_nch-1] le xmin,nind)
		if(nind gt 0) then begin
			ich0 = ind[nind-1]				; for the combined spectrum
		endif
	endelse
	;
	if((*PTR_main).right eq 0.0) then begin
		xmax = max(all_wave)
		(*PTR_main).right = xmax
	endif else begin
		xmax = (*PTR_main).right
		ind = WHERE((*PTR_main).spectrum_wave[0:spec_nch-1] ge xmax,nind)
		if(nind gt 0) then begin
			ich1 = ind[0]
		endif
	endelse
	;
	replot = 1
	if((*PTR_main).bottom eq 0.0) then begin
		if((*PTR_main).rescale_y_flag eq 1L) then ymin = MIN((*PTR_main).spectrum_flux[ich0:ich1]) else ymin = MIN(all_flux)
		(*PTR_main).bottom = ymin
		replot = 0
	endif else begin
		ymin = (*PTR_main).bottom
	endelse
	;
	if((*PTR_main).top eq 0.0) then begin
		if((*PTR_main).rescale_y_flag eq 1L) then ymax = MAX((*PTR_main).spectrum_flux[ich0:ich1]) else ymax = MAX(all_flux)
		(*PTR_main).top = ymax
		replot = 0
	endif else begin
		ymax = (*PTR_main).top
	endelse
	;
;	print, "Min, Max wavelegths = ",xmin, xmax
;	print, "Min, Max fluxes = ",ymin, ymax
;	if(ymin eq !VALUES.F_NAN or ymax eq !VALUES.F_NAN) then begin
;;	    print,"extreme flux values found...resetting"
;	    zz = median_filtered_mean(flux)
;	    ymin = zz[0] - 10.0 * zz[1]
;	    ymax = zz[0] + 10.0 * zz[1]
;	endif
	;
	;	if original spectrum, provide a bit of room at top and bottom,
	;	but if a replot, skip this
	;
	dy = (ymax - ymin)
	;
	if(replot eq 0) then begin
		ymax = ymax + 0.05 * dy
		ymin = ymin - 0.05 * dy
	endif
	;
	(*PTR_main).top = ymax
	(*PTR_main).bottom = ymin
	;
	;		plot limits found. Plot boundary, then loop through spectra
	;
	xxplot = [xmin, xmax]
	yyplot = [ymin, ymax]
	;
	plot, xxplot, yyplot, XRANGE=[xmin, xmax], YRANGE=[ymin, ymax], XSTYLE=1, $
		color=11, background=10, thick=1, YSTYLE=1, LINESTYLE=0, PSYM=10, $
		xtitle='Wavelength [microns]',/nodata
	;
	;   load plot vectors
	;
	xplot = (*PTR_main).spectrum_wave[0:this_nch-1]
	yplot = (*PTR_main).spectrum_flux[0:this_nch-1]
	splot = (*PTR_main).spectrum_sflux[0:this_nch-1]
	this_color = (*PTR_main).spectrum_color[0:this_nch-1]
	dx = xplot[1] - xplot[0]
	dx2 = dx / 2.
	;
	;	have to do one channel at a time, to enable color		;
	;
	for ich = 0, this_nch-2 do begin
		;
		xch = xplot[ich]+dx2*[-1.,1.,1.]
		ych = [yplot[ich],yplot[ich],yplot[ich+1]]
		sch = [splot[ich],splot[ich],splot[ich+1]]
		;
		oplot, xch, ych,thick=2, psym=-3,symsize=0.003, color=color_order[this_color[ich]]
		;
		;	if the flag for the flux uncertainty spectrum is also set, draw a light version of it
		;
		if((*PTR_main).plot_uncertainties eq 1L) then begin
			oplot, xch, sch, thick=1, color=get_color(Event, 'yellow')
		endif
		;
		;	trap, save plot limits
		;
	endfor
	;
	clip = !P.clip
	(*PTR_main).plot_x_offset = clip[0]					; starting x pixel
	(*PTR_main).plot_y_offset = clip[1]					; starting y pixel
	(*PTR_main).plot_x_size = clip[2] - clip[0] + 1		; x size of plot region
	(*PTR_main).plot_y_size = clip[3] - clip[1] + 1		; y size of plot region
	;
	;	plot scale factors
	;
	(*PTR_main).plot_x_scale = ( (*PTR_main).right - (*PTR_main).left ) / (*PTR_main).plot_x_size
	(*PTR_main).plot_y_scale = ( (*PTR_main).top - (*PTR_main).bottom ) / (*PTR_main).plot_y_size
	;
	;   if unity is between ymax and ymin, plot continuum line
	;
	if(ymax gt 1. and ymin lt 1. and (*PTR_main).plot_unity_line eq 1L) then begin
	    xz = xxplot
	    yz = [1., 1.]
	    oplot, xz, yz, color=get_color(event,'green'),linestyle=2
	endif
	;
	;	dashed zero line, if flag set and zero contained in plot
	;
	if(ymax gt 0. and ymin lt 0. and (*PTR_main).plot_zero_line eq 1L) then begin
	    xz = xxplot
	    yz = [0., 0.]
	    oplot, xz, yz, color=get_color(event,'blue'),linestyle=2
	endif
	;
	;	if any red or blue channels plotted, add legend
	;
	ind = WHERE( (*PTR_main).spectrum_color[0:(*PTR_main).spectrum_nch-1] eq 0 or $
				(*PTR_main).spectrum_color[0:(*PTR_main).spectrum_nch-1] eq 1,nrb)
	if(nrb gt 0) then begin
		xt = xxplot[0]+(xxplot[1]-xxplot[0])*0.50
		dxt = (xxplot[1]-xxplot[0])*0.05
		yt = yyplot[0]+(yyplot[1]-yyplot[0])*0.80
		dyt = (yyplot[1]-yyplot[0])*0.05
		;
		xyouts, xt, yt, 'Spectral Colors:',align=0.,charsize=1.75,charthick=2,$
					color=get_color(event,'white')
		xyouts, xt+dxt,yt-dyt,'White = Normal Channels',align=0.,charsize=1.5,$
					charthick=2,color=get_color(event,'white')
		xyouts, xt+dxt,yt-2.0*dyt,'Blue = No or Deleted Data',align=0.,charsize=1.5,$
					charthick=2,color=get_color(event,'blue')
		xyouts, xt+dxt,yt-3.0*dyt,'Red = Overlapped Input Data',align=0.,charsize=1.5,$
					charthick=2,color=get_color(event,'red')
		xyouts, xt+2.0*dxt,yt-3.8*dyt,'(lower flux uncert. chosen)',align=0.,$
					charsize=1.5,charthick=2,color=get_color(event,'red')
	endif
end
;
;-----------------------------------------------------------------
;
pro Toggle_Zero_Line, Event
	;
	;	toggle the status of the plot zero line radio button and buffer
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_main
	;
	(*PTR_main).plot_zero_line = 1L - (*PTR_main).plot_zero_line
	;
	wid = WIDGET_INFO(Event.top, FIND_BY_UNAME="Zero_Line_Button")
	WIDGET_CONTROL, wid, SET_BUTTON=(*PTR_main).plot_zero_line
	;
	;	replot the spectrum or spectra
	;
	if((*PTR_main).spectrum_ready eq 0) then begin
		plot_spectrum, event
	endif else begin
		plot_full_spectrum, event
	endelse
	;
end
;
;-----------------------------------------------------------------
;
pro Toggle_Unity_Line, Event
	;
	;	toggle the status of the plot unity line radio button and buffer
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_main
	;
	(*PTR_main).plot_unity_line = 1L - (*PTR_main).plot_unity_line
	;
	wid = WIDGET_INFO(Event.top, FIND_BY_UNAME="Unity_Line_Button")
	WIDGET_CONTROL, wid, SET_BUTTON=(*PTR_main).plot_unity_line
	;
	;	replot the spectrum or spectra
	;
	if((*PTR_main).spectrum_ready eq 0) then begin
		plot_spectrum, event
	endif else begin
		plot_full_spectrum, event
	endelse
end
;
;-----------------------------------------------------------------
;
pro toggle_active_offsets, Event
	;
	;	toggle the status of the active offset flag
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_main
	;
	(*PTR_main).active_offsets = 1L - (*PTR_main).active_offsets
	;
end
;
;-----------------------------------------------------------------
;
pro Clear_Offsets, Event
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_main
	;
	;	turn off offsetting and zero the offsets
	;
	(*PTR_main).active_offsets = 0
	;
	(*PTR_main).input_offset[*] = 0.0
	;
	;
	;	then, replot
	;
	plot_spectrum, Event
	;
end
